import _WidgetBase = require("./_WidgetBase");
import _OnDijitClickMixin = require("./_OnDijitClickMixin");
import _FocusMixin = require("./_FocusMixin");
import { _WidgetBaseConstructor } from "./_WidgetBase";

interface _Widget extends _WidgetBase, _OnDijitClickMixin, _FocusMixin {
    /**
     * Connect to this function to receive notifications of mouse click events.
     */
    onClick(event: DocumentEvent): void;

    /**
     * Connect to this function to receive notifications of mouse double click events.
     */
    onDblClick(event: DocumentEvent): void;

    /**
     * Connect to this function to receive notifications of keys being pressed down.
     */
    onKeyDown(event: DocumentEvent): void;

    /**
     * Connect to this function to receive notifications of printable keys being typed.
     */
    onKeyPress(event: DocumentEvent): void;

    /**
     * Connect to this function to receive notifications of keys being released.
     */
    onKeyUp(event: DocumentEvent): void;

    /**
     * Connect to this function to receive notifications of when the mouse button is pressed down.
     */
    onMouseDown(event: DocumentEvent): void;

    /**
     * Connect to this function to receive notifications of when the mouse moves over nodes contained within this widget.
     */
    onMouseMove(event: DocumentEvent): void;

    /**
     * Connect to this function to receive notifications of when the mouse moves off of nodes contained within this widget.
     */
    onMouseOut(event: DocumentEvent): void;

    /**
     * Connect to this function to receive notifications of when the mouse moves onto nodes contained within this widget.
     */
    onMouseOver(event: DocumentEvent): void;

    /**
     * Connect to this function to receive notifications of when the mouse moves off of this widget.
     */
    onMouseLeave(event: DocumentEvent): void;

    /**
     * Connect to this function to receive notifications of when the mouse moves onto this widget.
     */
    onMouseEnter(event: DocumentEvent): void;

    /**
     * Connect to this function to receive notifications of when the mouse button is released.
     */
    onMouseUp(event: DocumentEvent): void;

    postCreate(): void;

    /**
     * Deprecated.  Use set() instead.
     */
    setAttribute(attr: string, value: any): void;

    /**
     * This method is deprecated, use get() or set() directly.
     */
    attr(name: string | { [attr: string]: any }, value?: any): any;

    /**
     * Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.
     */
    getDescendants(): _Widget[];

    /**
     * Called when this widget becomes the selected pane in a
     * `dijit/layout/TabContainer`, `dijit/layout/StackContainer`,
     * `dijit/layout/AccordionContainer`, etc.
     *
     * Also called to indicate display of a `dijit.Dialog`, `dijit.TooltipDialog`, or `dijit.TitlePane`.
     */
    onShow(): void;

    /**
     * Called when another widget becomes the selected pane in a
     * `dijit/layout/TabContainer`, `dijit/layout/StackContainer`,
     * `dijit/layout/AccordionContainer`, etc.
     *
     * Also called to indicate hide of a `dijit.Dialog`, `dijit.TooltipDialog`, or `dijit.TitlePane`.
     */
    onHide(): void;

    /**
     * Called when this widget is being displayed as a popup (ex: a Calendar popped
     * up from a DateTextBox), and it is hidden.
     * This is called from the dijit.popup code, and should not be called directly.
     *
     * Also used as a parameter for children of `dijit/layout/StackContainer` or subclasses.
     * Callback if a user tries to close the child.   Child will be closed if this function returns true.
     */
    onClose(): boolean;
}

type _WidgetConstructor = _WidgetBaseConstructor<_Widget>;
declare const _Widget: _WidgetConstructor;
export = _Widget;
